[#]: subject: (Sustainable economic development begins with open thinking)
[#]: via: (https://opensource.com/open-organization/21/3/sustainable-development-environment)
[#]: author: (Ron McFarland https://opensource.com/users/ron-mcfarland)
[#]: collector: (lujun9972)
[#]: translator: ( )
[#]: reviewer: ( )
[#]: publisher: ( )
[#]: url: ( )

Sustainable economic development begins with open thinking
======
Do our global development practices respect planetary boundaries?
Greater transparency could provide insight.
![2 cents penny money currency][1]

To be successful, open organizations must have specific purposes, address achievable goals, perform clear tasks, effectively evaluate the results of their work, and introduce countermeasures or revisions to their operations. [Open organization principles][2] serve vital functions throughout this process.

This is true no matter the scale of the problem an open organization confronts. In this three part review of [_The Age of Sustainable Development_][3] by Jeffrey Sachs, I'll examine an issue with global scope—sustainable economic development—to demonstrate how thinking openly can help us address global issues.

Specifically, in this article I'll discuss the important role of _transparency_ in assessing environmental damage and destruction some economic development programs can cause, and I'll explain how the principle of transparency also helps us think about specific actions we can take to combat destructive forces. In the next article, I will discuss human suffering on a global scale—here again stressing transparency. And in the third and last article, I will bring both these concerns together and discuss global governance. That is where we'll recognize the importance of open organization principles for making progress on these issues.

One more note: All these articles address extremely complex subjects. So at the end of each, I'll share a video presentation offering additional explanation. I'll start that discussion by introducing what I call the Open Organization Principles Loop, so you can visualize how these principles can be applied to sustainable global economic development issues.

Simply put, letting [open principles][2] guide our work creating sustainable, global economic development will help:

  * Make global, environmental and human suffering problems **transparent**
  * Form and unite organizations/**communities** at the local (not only global) level to tackle those problems
  * Start **collaboration** within and between communities to find detailed, local solutions
  * Recruit and **include** members globally to gain broad perspectives on how to achieve goals
  * **Adapt** strategies to each region globally and in each local community



All this starts with exposing specific problems and making them vividly transparent. Once exposed, these problems must be broadly presented and made _immediately personal_ to every individual.

This is the discussion I hope to start in this article on global environmental issues.

The earth has what we might call "planetary boundaries." Respecting these boundaries means being prosperous, socially inclusive, and environmentally responsible.

### Stressing planetary boundaries

In a previous article entitled "[Climate challenges call for open solutions][4]," I discussed only the issue of carbon-free power generation, particularly through fourth-generation nuclear power plants that are now being developed. But reading Sach's book, _The Age of Sustainable Development_, made me realize that energy generation through nuclear power plants is an approach _too narrow_ to address the massive climate challenges the world faces (the phrase "climate challenge" might _itself_ be too narrow). "Sustainable economic development" might be a better phrase to describe what should be our most pressing concern, of which climate change is only _one_ component issue. When we use the term "sustainable," we're referring to methods that effectively avoid driving something or some species to extinction—including humankind. It is the conservation of living and mineral resources in ways that continue to make life on earth economically viable.

Sachs' book suggests a wide range of strategies on detailed concerns that could move the global society toward sustainable economic development. I will quickly review them and explain where open organization principles can play a role.

The earth has what we might call "[planetary boundaries][5]": it can only support life to the extent that life forms respect these boundaries. They are the foundations of ecosystems like forests and fisheries, for example. Respecting these boundaries means being prosperous, socially inclusive, and environmentally responsible.

We must monitor and manage our relationships with these planetary boundaries globally. Here are at least nine boundaries:

  1. **Climate change**: This is directly related to [greenhouse gases][6] (GHGs).
  2. **Ocean acidification**: Rising acidity threatens various marine life.
  3. **Stratospheric ozone depletion**: Evidence suggests a direct link to skin cancers and other disorders.
  4. **Biogeochemical flow boundaries (Nitrogen Cycle/Phosphorus Cycle)**: Chemical fertilizers are required for high crop yield, but their runoff negatively impacts the surrounding environment.
  5. **Global freshwater use and water scarcity**: Groundwater is declining worldwide while demand is growing with population growth.
  6. **Land degradation**: Deforestation is a major problem, as forests withdraw CO2 in the atmosphere and are the habitat of many species.
  7. **Biodiversity loss**: We depend on biodiversity for our food supply, for our safety from many natural hazards like flooding, industry/construction material supply, freshwater and combating pests and pathogens.
  8. **Atmospheric aerosol loading**: Simply put, smog causes problems like life-threatening lung disease.
  9. **Chemical pollution**: Petrochemical production, steel production, and mining processes put deadly pollutants into their surrounding environments.



Sachs indicates that humans have stressed these boundaries in major areas around the globe, and earth's natural system can't cope with that pressure. Sachs believes that during this century human beings will stress all nine boundaries in critically dangerous ways, unless dedicated organizations and communities are formed to establish projects that execute on reversing them.

### Why can't we just save the world?

It's difficult to convince people of environmental importance and to assemble organizations, communities, neighborhoods and resources to execute action plans. Sachs cites six primary reasons for this:

  1. **Global problems.** No single actor can take charge to get things started and organizing on a global scale is extremely difficult.
  2. **Short-term special interests groups.** Many parties have competing goals, and no one is [speaking for the planet][7].
  3. **Impact timeframe.** The greatest impact will be on those not born yet.
  4. **Current economic structure.** The solutions might be incompatible with the current economic structure. But, its damage to the environment must be exposed and a new sustainable economic structure technically developed to replace it.
  5. **Pace of change.** The impact is not visible on a daily, weekly, or even monthly basis.
  6. **Complexity.** Environmental issues are caused by many human activities, not just one.



Reviewing these six characteristics, we see that getting solutions adopted globally requires a powerful sales job. But applying open organization principles can be significantly helpful, particularly making the problems as transparent as possible on a global scale.

Both globally and locally, communities can't continue with their current modes of economic development—not if they want to respect planetary boundaries. Business as usual, Sachs stresses, will destroy us all.

As I mentioned earlier, Sachs notes that generally addressing sustainable development directly is too vague and unhelpful. The challenge must be broken down into detailed projects and tasks for each region and community on the planet. Here are some specific examples of initiatives that are important but could vary by region:

  1. Building/home energy use reduction
  2. Environmentally friendly transportation
  3. Secure food production to reach demand
  4. Clean/efficient electrical power generation, storage and distribution
  5. Environmentally friendly urban designing
  6. Ocean biodiversity protection
  7. Plant biodiversity protection
  8. Wildlife biodiversity protection
  9. Air pollution &amp; the CO2 reduction
  10. Freshwater supply to satisfy future demand



### The cost of doing nothing

Both globally and locally, communities can't continue with their current modes of economic development—not if they want to respect planetary boundaries. Business as usual, Sachs stresses, will destroy us all.

At the start of this discussion, I mentioned the importance of transparency and the need to expose the environmental damage that many human economic activities cause. Information and communication technology specialists will become increasingly central to these transparency efforts, as they can effectively and cost efficiently use automated data gathering, analytics, telecommunications, and other technologies to help increase transparency. (Wondering how you might do this? Why not [ask our community][8]?)

In the next article, I will attempt to make vividly transparent the impact of unsustainable development practices on human inequality and suffering at a global level. Armed with a full understanding of both these challenges, in my last article in this series, I'll present global governance to tackle them. In that governance all open organization principles will play vital roles.

--------------------------------------------------------------------------------

via: https://opensource.com/open-organization/21/3/sustainable-development-environment

作者：[Ron McFarland][a]
选题：[lujun9972][b]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]: https://opensource.com/users/ron-mcfarland
[b]: https://github.com/lujun9972
[1]: https://opensource.com/sites/default/files/styles/image-full-size/public/lead-images/Medical%20Costs%20Transparency_1.jpg?itok=CkZ_J88m (2 cents penny money currency)
[2]: https://theopenorganization.org/definition/
[3]: https://www.goodreads.com/book/show/23215948-the-age-of-sustainable-development
[4]: https://opensource.com/open-organization/19/10/global-energy-climate-challenges
[5]: http://www.igbp.net/news/features/features/aplanetontheedge.5.1b8ae20512db692f2a680003122.html
[6]: https://en.wikipedia.org/wiki/Greenhouse_gas#/media/File:Global_GHG_Emissions_by_Sector_2016.png
[7]: https://opensource.com/open-organization/19/10/open-platform-greenpeace
[8]: http://theopenorganization.community
