[#]: collector: (lujun9972)
[#]: translator: ( )
[#]: reviewer: ( )
[#]: publisher: ( )
[#]: url: ( )
[#]: subject: (Setting a standard for digital public goods)
[#]: via: (https://opensource.com/article/20/12/digital-public-goods)
[#]: author: (Lucy https://opensource.com/users/lucyeoh)

Setting a standard for digital public goods
======
Nine-indicator standard aims to promote open source software, data, AI
models, standards, and content for a more equitable world.
![Two diverse hands holding a globe][1]

In June 2020, the Secretary-General of the United Nations published a "[Roadmap for Digital Cooperation][2]." In this report, he expanded on recommendations made a year before, calling on all actors, including the Member States, the United Nations system, the private sector, and others, to promote digital public goods. He says to realize the benefits of increased internet connectivity, open source projects in the form of digital public goods must be at the center.

While the term "digital public good" appears as early as April 2017, this report offers the first broadly accepted definition of digital public goods:

> "Open source software, open data, open AI models, open standards, and open content that adhere to privacy and other applicable international and domestic laws, standards and best practices, and do no harm."

The Digital Public Goods Alliance (DGPA) translated that definition into a [nine-indicator open standard][3] that we hope will serve as a comprehensive, shared definition to promote the discovery, development, use of, and investment in digital public goods for a more equitable world.

The DPG standard was developed from the original 51-indicator standard used by the DPGA in the preliminary review of [Early Grade Reading][4] projects and refined through contributions from many experts. The standard is itself an open source project. The core version [lives on GitHub][5] and is open to contributions, revisions, and suggestions from anyone. Proposed revisions are regularly reviewed in alignment with the standard's [governance principles][6], and we invite anyone who uses and benefits from the standard to join our growing [list of endorsers][7].

The Digital Public Goods Alliance envisions engaging a growing community of contributors and stakeholders around the [Digital Public Goods Standard][3]. Our ambition is to balance responsiveness to feedback with stability and predictability for the standard so that it can be a reliable framework that supports development, recognition, and advocacy for digital public goods.

### Digital Public Goods Standard

Below is the DPG Standard's full list of indicators and requirements in v.1.1.2. These must be met in order for a nominated project (software, data, AI model, standard, and/or piece of content) to be considered a digital public good.

**Indicator**

**Requirement**

1\. Relevance to Sustainable Development Goals

All projects must indicate the [Sustainable Development Goals][8] (SDGs) that they are relevant to and provide supporting links/documentation to support this relevance.

2\. Use of approved open source license

Projects must demonstrate the use of an approved open source license. For open source software, we only accept [OSI approved licenses][9]. For open content, we require the use of a [Creative Commons license][10], while we encourage projects to use a license which allows for both derivatives and commercial reuse ([CC-BY][11] and [CC-BY-SA][12]), or dedicate content to the public domain ([CC0][13]); we also accept licenses which do not allow for commercial reuse ([CC BY-NC][14] and [CC BY-NC-SA][15]). For data, we require an [Open Data Commons approved license][16]. You can find [the full license list here][17].

3\. Documentation of ownership

Ownership of everything that the project produces must be clearly defined and documented, i.e., through copyright, trademark, or other publicly available information.

4\. Mandatory dependencies

If the open source project has mandatory dependencies that create more restrictions than the original license, the projects must be able to demonstrate independence from the closed component and/or indicate the existence of functional, open alternatives.

5\. Documentation

The project must have some documentation of the source code, use cases, and/or functional requirements. For content, this should indicate any relevant compatible apps, software, hardware required to access the content, and instructions about how to use it. For software projects, this should be present as technical documentation that would allow a technical person unfamiliar with the project to launch and run the software. For data projects, this should be present as documentation that describes all the fields in the set and provides context on how the data was collected and how it should be interpreted.

6\. Mechanism for extracting data

If this project has non personally identifiable information, there must be a mechanism for extracting or importing non personally identifiable information (PII) data from the system in a non-proprietary format.

_Note that evidence for requirements 7 through 9 can only be given by someone authorized to speak on behalf of the project. We collect title, name, and contact information to confirm this authority._

7\. Adherence to privacy and applicable laws

The project must state that, to the best of its knowledge, it complies with relevant privacy laws and all applicable international and domestic laws.

8\. Adherence to standards and best practices

Projects must demonstrate some adherence to standards, best practices, and/or principles, i.e., the principles for digital development.

9\. Do no harm

All projects must demonstrate that they have taken steps to ensure that the project anticipates, prevents, and does no harm.

9a. Data privacy and security

Projects that collect data must identify the types of data collected and stored and demonstrate that the project ensures the privacy and security of this data and has taken steps to prevent adverse impacts resulting from its collection, storage, and distribution.

9b. Inappropriate and illegal content

Projects that collect, store, or distribute content must have policies identifying inappropriate and illegal content such as child sexual abuse materials and mechanisms for detecting, moderating, and removing inappropriate/illegal content.

9c. Protection from harassment

If the project facilitates interactions with or between users or contributors, there must be a mechanism for users and contributors to protect themselves against grief, abuse, and harassment. The project must have a mechanism to address the safety and security of underage users.

If you know of a project that you believe meets these standards and should be considered a digital public good, please [submit it][18]. We also welcome your feedback on the standard in the comments below.

* * *

_This article is adapted with permission from the post "Setting a standard for digital public goods," which originally appeared on the Digital Public Goods Alliance_ [_blog_][19] _and_ [_Medium_][20] _channel._

--------------------------------------------------------------------------------

via: https://opensource.com/article/20/12/digital-public-goods

作者：[Lucy][a]
选题：[lujun9972][b]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]: https://opensource.com/users/lucyeoh
[b]: https://github.com/lujun9972
[1]: https://opensource.com/sites/default/files/styles/image-full-size/public/lead-images/world_hands_diversity.png?itok=zm4EDxgE (Two diverse hands holding a globe)
[2]: https://www.un.org/en/content/digital-cooperation-roadmap/
[3]: https://digitalpublicgoods.net/standard/
[4]: https://medium.com/digital-public-goods/announcing-the-first-vetted-digital-public-goods-for-foundational-literacy-and-early-grade-reading-1f5c371a50d3
[5]: https://github.com/DPGAlliance/DPG-Standard/blob/master/standard.md
[6]: https://github.com/DPGAlliance/DPG-Standard/blob/master/governance.md
[7]: https://github.com/DPGAlliance/DPG-Standard/blob/master/endorsement.md
[8]: https://sdgs.un.org/goals
[9]: https://opensource.org/licenses
[10]: https://creativecommons.org/licenses/
[11]: https://creativecommons.org/licenses/by/4.0/
[12]: https://creativecommons.org/licenses/by-sa/4.0/
[13]: https://creativecommons.org/choose/zero/
[14]: https://creativecommons.org/licenses/by-nc/4.0/
[15]: https://creativecommons.org/licenses/by-nc-sa/4.0/
[16]: https://opendefinition.org/licenses/
[17]: https://github.com/unicef/publicgoods-candidates/blob/master/docs/licenses.md
[18]: https://forms.gle/8w4pXRUqUV2oKAHi9
[19]: https://digitalpublicgoods.net/blog/setting-a-standard-for-digital-public-goods/
[20]: https://medium.com/digital-public-goods/setting-a-standard-for-digital-public-goods-a1f28b3d1ed5
