[#]: subject: (Building open organizations to make a better life more sustainable for everyone)
[#]: via: (https://opensource.com/open-organization/21/5/sustainable-development-human-impacts)
[#]: author: (Ron McFarland https://opensource.com/users/ron-mcfarland)
[#]: collector: (lujun9972)
[#]: translator: ( )
[#]: reviewer: ( )
[#]: publisher: ( )
[#]: url: ( )

Building open organizations to make a better life more sustainable for everyone
======
By opening our approach to ensuring prosperity for all, we can advance
more sustainable solutions.
![Lots of hands trying to climb a ladder][1]

In the first article in this series reviewing [_The Age of Sustainable Development_ by Jeffrey Sachs][2], I discussed the impact of economic development on the environment, and I explained how [open organization principles][3] can help us begin building sustainable, global economic development plans for the future.

In this article, I will continue to explore Sach's argument about the future of sustainable development. This time, I'll address development issues Sachs says are impacting humans, and I'll offer a few ideas about how we might leverage open organization principles to begin addressing those issues. As I did previously, I'll conclude with a video in which I offer more detail and further explanation.

### Confronting poverty

_Why do some countries still experience extreme poverty?_

This is the first question one must ask when exploring human suffering that might occur as a result of unsustainable economic development. Simply by asking this question, we bring the issue to light, making problems (and solutions) more _transparent_.

"[Extreme poverty][4]" is defined by a lack of basic human needs, such as food, clean water, shelter, sanitation, clothing, health care, safe energy, transportation, education, and connectivity. Some regions of the world experience this more than others. Why? And could thinking with open organization principles help us expose the reasons?

Sachs introduced some of these reasons in another of his books, [_The End of Poverty_][5]:

  1. **Extreme poverty trap:** Emerging from extreme poverty requires a certain degree of investment; a country may not have the resources to provide the basic investment required to escape extreme poverty.
  2. **Bad economic policy:** A regional or national government selects the wrong economic policy for the situation.
  3. **Governmental expertise:** The government lacks the resources to explore requisite physical infrastructure (roads, schools, clinics), or "soft" infrastructure (hiring doctors, teachers, or engineers).
  4. **Physical geography:** The country is landlocked ([Africa][6] has the most landlocked countries in the world with 16 out of 55 countries), is in remote locations high in the mountains (not suitable for farming or low-cost manufacturing), does not have valuable minerals/energy raw materials/wind/direct sunlight, is susceptible to diseases (malaria, Zika, AIDS), or experiences natural disasters (earthquakes, tsunamis, tropical cyclones, droughts). Here Sachs references the [Koppen-Geiger (K-G) map][7] to classify climates globally to determine each region's best strategy for building a sustainable economy.
  5. **Poor policy execution:** The region had selected the right policies but failed to execute on them due to corruption or incompetence.
  6. **Cultural barriers:** Cultural values and norms inhibit development (for example, [systemic discrimination against women and girls][8]), or barriers between ethnic groups (like racial or nationality differences) have caused problems.
  7. **Geopolitics:** The region has security issues between neighboring countries.



Anyone who wants to create an open organization to address these issues must decide on the purpose and goals for that community. Developing that strong sense of purpose is only possible when everyone knows the "situation on the ground"—that is, when everyone is clear about the problem's they're facing—and has some sense of the history of which tactics have worked and which haven't. Only when the community has a clear and compelling sense of purpose can all the open organization principles go to work.

Anyone who wants to create an open organization to address these issues must decide on the purpose and goals for that community.

### Where to begin?

In _The Age of Sustainable Development_, Sachs offers detailed suggestions for addressing problems like these (including suggestions for targeted funding). Breaking down the problems in this way, we can also more easily see how we might introduce open organization principles to address them.

Sachs asks readers to look closely at six factors impacting countries facing extreme poverty and suggests economic development strategies he considers sustainable:

  1. **For landlocked regions:** Build a transportation system to the nearest major ocean or river port, build close relationships with coastal neighbors, develop internet access, and emphasize export activities. Building an open organization to address extreme poverty in a landlocked country would involve engaging with stakeholders who can access a port.
  2. **For water-stressed regions:** Build water irrigation and purification systems powered by solar/wind energy for farmers. Develop new crops that do not require huge amounts of water. This could improve food stability. An open organization community addressing this challenge would need to begin by considering how to get that country the water it needs, as well as offering new crop strategies.
  3. **For regions with a heavy disease burden:** Build public health programs for disease control and collaborate with international health agencies. Also, some kind of [community health worker][9] might be helpful. If disease is the main issue, maybe an open organization community project on disease control should be explored.
  4. **For regions impacted by natural hazards:**Develop physical and social infrastructure to help the public prepare for increased probabilities of floods, cyclones, and extreme storm events. Also, develop mass migration strategies to relocate populations from regions that will surely become uninhabitable in the years ahead. If natural hazards, like flooding, are the primary issues a country faces, then perhaps an open organization community project addressing those hazards should be explored.
  5. **For regions lacking major energy sources:** Develop alternative energy sources (geothermal, hydro, wind, solar, nuclear power). In regions lacking major energy sources, introduce clean energy sources. For regions heavily reliant on fossil fuel resources, consider implementing programs and policies that adequately distribute wealth throughout the entire society. Furthermore, if these regions are involved in burning, introduce some kind of carbon capture system. If energy availability is the primary issue a region is facing, then maybe an open organization project concerning power supplies should be explored.
  6. **For regions requiring more education opportunities:** Examine factors like demographics and [total fertility rate (TFR)][10] to develop strategies for helping families in the region become economically sustainable. According to Sachs, greater and better educational opportunities lead to better health, better nutrition per child, and more manageable population growth. If education and human capital development are the issues at hand, maybe an open organization project exploring new educational opportunities, and strategies for virtual learning should be explored.



Considering all these issues, Sachs recommends more targeted [official development assistance][11] in agriculture, health, education/skills creation, infrastructure, and women's empowerment. Also, new methods of funding should be explored, like [Partners for Development][12]. This must be very carefully done, Sachs warns, as poorly directed funds will not produce the desired results.

### Population explosion

Without any consideration of the world's growing human population, Sachs notes, we can't address other global sustainable economic development issues. The [global population][13] is growing, as is the [per capita productivity and consumption][14]. The [population has grown][15] by _nine times_ the 800 million people estimated to have lived around the start of the Industrial Revolution. Simply put, this means the world is full of the 7.8 billion people seeking economic improvement (leading to greater resource consumption per person). And right now, the world economy is estimated to produce $90 trillion of output per year (the Gross World Product, [GWP][16]). So there must be investments in agriculture, health, education/skills creation, and physical infrastructure, as well as some consideration of global fertility rates, otherwise [extreme poverty][4] will never be eradicated.

Without any consideration of the world's growing human population, Sachs notes, we can't address other global sustainable economic development issues.

To make matters worse, population growth stresses global supplies of water, nitrogen, and carbon, upon which life depends. This is unprecedented in the span of [humanity’s 10,000 years of civilization][17]. Therefore, Sachs recommends sustainable, holistic development that encompasses all economic, social, and environmental concerns.

Reviewing the concerns above, we can see four basic issues:

  1. **Economic prosperity:** Any proposal regarding the above issues must take the global economy into consideration. We must be monitoring economic changes, whether locally or globally, long-term or short-term. Some industries will decline, but they must be replaced with more environmentally friendly industries, so communities can experience economic stability.
  2. **Environmental sustainability:** I discussed this in the [first part of my review][18]. Human activities should not adversely impact the local and global natural environment.
  3. **Social inclusion and cohesion:** This is the improvement of human life and getting rid of extreme poverty on the planet.
  4. **Governance:** I'll talk about this more in the next and final part of my review. Whether it be governments, businesses, community organizations, social support, neighborhood networks within communities, schools, families, parents, or others, supervision and management of economically sustainable policies and practices will be important.



In the first article of this review series, I've explained Sach's take on development issues impacting the global environment. In this article, I focused primarily on the importance of transparency and awareness of human suffering globally. We can achieve this kind of transparency efficiently and cost-effectively by turning to information and communication technology specialists using automated data gathering and analytics, utilizing telecommunication tools, and social media. For example, sensors and communications devices are now being developed and deployed to measure water requirements and nutrients in soil. These data can be gathered, distributed among specialists, and can inform collaborastors working on solutions. Even now, this process is helping both to reduce environmental fertilizer runoff damage and improve crop production. 

In the final article in this series, I'll discuss global actions we can take to address these challenges. They begin with the issue of global governance.

--------------------------------------------------------------------------------

via: https://opensource.com/open-organization/21/5/sustainable-development-human-impacts

作者：[Ron McFarland][a]
选题：[lujun9972][b]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]: https://opensource.com/users/ron-mcfarland
[b]: https://github.com/lujun9972
[1]: https://opensource.com/sites/default/files/styles/image-full-size/public/lead-images/BUSINESS_heirarchy.png?itok=ExGiv98I (Lots of hands trying to climb a ladder)
[2]: http://cup.columbia.edu/book/the-age-of-sustainable-development/9780231173155
[3]: https://theopenorganization.org/definition/
[4]: https://www.worldbank.org/en/news/feature/2016/06/08/ending-extreme-poverty#:~:text=The%20World%20Bank%20defines%20%E2%80%9Cextreme,extreme%20poverty%20can%20be%20achieved.
[5]: https://en.wikipedia.org/wiki/The_End_of_Poverty
[6]: https://www.thoughtco.com/african-countries-that-are-landlocked-4060437#:~:text=Out%20of%20Africa's%2055%20countries,Uganda%2C%20Zambia%2C%20and%20Zimbabwe.
[7]: http://koeppen-geiger.vu-wien.ac.at/present.htm
[8]: https://www.worldbank.org/en/research/dime/brief/dime-gender-program
[9]: https://www.who.int/hrh/documents/community_health_workers_brief.pdf
[10]: https://ourworldindata.org/grapher/children-per-woman-un
[11]: https://en.wikipedia.org/wiki/Official_development_assistance
[12]: http://pfd.org/
[13]: https://www.worldometers.info/world-population/
[14]: https://en.wikipedia.org/wiki/List_of_countries_by_GDP_(PPP)_per_capita
[15]: https://ourworldindata.org/grapher/world-population-since-10000-bce-ourworldindata-series
[16]: https://en.wikipedia.org/wiki/Gross_world_product#:~:text=The%20gross%20world%20product%20(GWP,gross%20domestic%20product%20(GDP).
[17]: https://opensource.com/open-organization/20/8/global-history-collaboration
[18]: https://opensource.com/open-organization/21/3/sustainable-development-environment
