[#]: subject: (Accessibility in open source for people with ADHD, dyslexia, and Autism Spectrum Disorder)
[#]: via: (https://opensource.com/article/21/7/open-source-neurodiversity)
[#]: author: (Rikard Grossman-Nielsen https://opensource.com/users/rikardgn)
[#]: collector: (lujun9972)
[#]: translator: ( )
[#]: reviewer: ( )
[#]: publisher: ( )
[#]: url: ( )

Accessibility in open source for people with ADHD, dyslexia, and Autism Spectrum Disorder
======
Open source accommodations help people with neurodiversity use their
talents to their highest ability.
![a magnifying glass looking at a brain illustration][1]

For a long time, attention deficit hyperactivity disorder (ADHD), autism, Asperger syndrome, dyslexia, and other neurodiverse conditions were considered things that hold people back. But now, many researchers and employers recognize that [neurodiversity is a competitive advantage][2], especially in technology, and especially when certain accommodations are provided.

This is certainly true for me. I'm a 39-year-old teacher in Sweden diagnosed with ADHD and Asperger's (also referred to as Autism Level 1). I'm also an intermediate Linux user and use it daily for Java programming, productivity, and gaming. I've been using Linux since the late 1990s, and I've learned ways open source programs can be made more accessible for people with these conditions. For example, I use accessibility software, including speech synthesis to find spelling errors and calendar software accommodations to help with my Asperger's and ADHD.

### Asperger's, ADHD, and dyslexia

Before I get into open source software accommodations, I'll share some information about these conditions.

#### Asperger syndrome

[Asperger's][3] is a form of autism without intellectual disability. People with Asperger's often have:

  * Difficulties in social contact with other people
  * Special interest areas that may consume a large part of their attention and time
  * Difficulties understanding and using language in communicating with other people
  * Deficits in motor skills
  * A tendency to easily get caught in up certain routines or actions
  * Unusual perception of and sensitivity to stimuli such as sound, light, etc.



#### ADHD

The three core symptoms of [ADHD][4] are:

  * **Attention:** Difficulties in concentration and forgetfulness; distractability; easily bored and fail to complete things that don't interest them
  * **Impulsivity:** Strong emotional reactions to different things; difficulty listening to others; problems handling unstructured situations that require reflection and thinking things through; sometimes impulsivity may lead to difficulties in motor control
  * **Hyperactivity:** Difficulty regulating activity to an appropriate level for the situation; trouble sitting still and winding down, possibly mixed with periods of exhaustion



Hyperactivity in children is often physical; in adults, it's more of an internal restlessness that might cause sleeping problems (among other things). Some people with ADHD have one of the three core symptoms, and others have two or all of them.

#### Dyslexia

Some people with neurodiverse conditions also have problems with reading and writing. This might be related to difficulties in attention, hyperactivity, and impulsivity. However, they might also be caused by [dyslexia][5].

  * People with dyslexia have difficulty recognizing and understanding words. They might place letters in the incorrect order, making reading comprehension more difficult.
  * Dyslexia isn't correlated with intelligence.
  * Dyslexia can't be cured, but accommodations can help a great deal in school and work.
  * Reading a lot and listening to audiobooks can improve the ability of people with dyslexia to read and write.



### Asperger's and ADHD at work

While the symptoms associated with Asperger's and ADHD can make some parts of work challenging, other aspects give neurodiverse people advantages in the workplace.

#### Asperger's

Some of the skills people with [autism spectrum disorders bring to the workplace][6]:

  * High concentration power and precision in work
  * Attention to minute details
  * Patience for repetitive tasks
  * Higher memory power (can remember tiny details)
  * Hard-working
  * Loyal



#### ADHD

Some of the skills people with [ADHD bring to the workforce][7]:

  * Able to find unique solutions to difficult problems
  * Can talk about many different topics at one time
  * Good in a crisis; some of the most stressful jobs are staffed by those with ADHD
  * Empathetic and intuitive
  * Entrepreneurial



### Making software more accessible

The descriptions above are highly generalized and may not apply to all adults with Asperger's, ADHD, and dyslexia. One problem with current accessibility standards is that they confuse different neurodiversities. For example, they may not differentiate between autism with and without intellectual disability, the latter of which is called Asperger's or Autism Level 1, or they may assume dyslexia is an intellectual disability.

In his article [_User interface for people with autism spectrum disorders_][8], Nikolay Pavlov provides some suggestions to improve UI design:

  * Use simple graphics
  * Strive for simple, clear navigation
  * Do not use complex menus



People with Asperger's have different needs, abilities, and preferences, so these accommodations won't be beneficial to everyone. These UI features could also help people who have autism with intellectual disability, ADHD, dyslexia without intellectual disability, and other conditions. Therefore, when considering making accommodations in software, think carefully about your target group. And know that if you ask people for input, you will probably get many different answers.

People with ADHD especially might benefit from one of Pavlov's other recommendations:

  * Use visual indicators for time-consuming actions



This is valuable when people perceive that an app or web page is not loading quickly enough. I appreciate when systems give continuous feedback on their progress because it tells me that everything is in working order.

### Examples of accessibility

The GNOME calendar offers a good example of making software more accessible.

Compare the standard date view:

![Standard GNOME calendar][9]

(Rikard Grossman-Nielsen, [CC BY-SA 4.0][10])

To this modified view:

![Modified GNOME calendar][11]

(Rikard Grossman-Nielsen, [CC BY-SA 4.0][10])

It's a lot easier to find the marked date with the yellow circle around the number 29.

In contrast, [Vi and Vim][12] are among the least accessible text editors I've ever used, but note they aren't designed with accessibility in mind. My biggest problem is that that they don't offer any cues to their different commands. When I use a terminal editor, I prefer [Nano][13] because it provides cues to what keyboard commands to use. Most often, I use a graphical user interface (GUI) editor like [Gedit][14] or Nedit because it's easier for me to create text in a GUI editor.

### How GNOME embraces diversity

I've found that [GNOME][15] is the best of the large Linux desktop environments for offering accessibility features, but it can definitely still improve. Because I'm interested in Linux on the desktop and making it even more accessible, I joined the team planning [GUADEC][16], the GNOME Users And Developers European Conference.

GUADEC embraces a climate of diversity, both in theory and practice. The conference provides accommodations, such as captioning for online lectures and quiet rooms at physical events. The 2021 conference, to be held online July 21–25, will have a few lectures on UI design, and I will offer a workshop on [making open source applications more accessible][17]. If you'd like to learn more, visit the GUADEC website and [register][18].

--------------------------------------------------------------------------------

via: https://opensource.com/article/21/7/open-source-neurodiversity

作者：[Rikard Grossman-Nielsen][a]
选题：[lujun9972][b]
译者：[译者ID](https://github.com/译者ID)
校对：[校对者ID](https://github.com/校对者ID)

本文由 [LCTT](https://github.com/LCTT/TranslateProject) 原创编译，[Linux中国](https://linux.cn/) 荣誉推出

[a]: https://opensource.com/users/rikardgn
[b]: https://github.com/lujun9972
[1]: https://opensource.com/sites/default/files/styles/image-full-size/public/lead-images/LAW_EvidencedBasedIP_520x292_CS.png?itok=mmhCWuZR (a magnifying glass looking at a brain illustration)
[2]: https://hbr.org/2017/05/neurodiversity-as-a-competitive-advantage
[3]: https://en.wikipedia.org/wiki/Asperger_syndrome
[4]: https://en.wikipedia.org/wiki/Attention_deficit_hyperactivity_disorder
[5]: https://en.wikipedia.org/wiki/Dyslexia
[6]: https://www.thehrdigest.com/autistic-workers-strength-not-weakness/
[7]: https://adhdatwork.add.org/potential-benefits-of-having-an-adhd-employee/
[8]: https://www.researchgate.net/publication/276495184_User_Interface_for_People_with_Autism_Spectrum_Disorders
[9]: https://opensource.com/sites/default/files/uploads/gnome-calendar-standard.png (Standard GNOME calendar)
[10]: https://creativecommons.org/licenses/by-sa/4.0/
[11]: https://opensource.com/sites/default/files/uploads/gnome-calendar-modified.png (Modified GNOME calendar)
[12]: https://opensource.com/resources/what-vim
[13]: https://opensource.com/article/20/12/gnu-nano
[14]: https://opensource.com/article/20/12/gedit
[15]: https://opensource.com/downloads/cheat-sheet-gnome-3
[16]: https://events.gnome.org/event/9/
[17]: https://events.gnome.org/event/9/contributions/240/
[18]: https://events.gnome.org/event/9/registrations/34/
